#'pt' berechnet im Falle einer t-Verteilung mit 
#Nichtzentralitaetsparameter nonc (default 0) 
#die Wahrscheinlichkeit, einen Wert 
#kleiner oder gleich t zu messen.
#

twert1 <- #berechnet den t-Wert fuer mu_0 = 0
  function(mean,N,s) 
  {
    t <- mean*(sqrt(N))/s
    print(paste0("t-Wert = ",round(t,4)))
    t
  }
lot1 <- #vgl. Smithson, M. Nonct.ssc, http://www.michaelsmithson.online/stats/CIstuff/Nonct.ssc (Aufgerufen am 04.03.2019). 
  function(tval,df,conf)
  {
    ulim <- 1 - (1-conf)/2 # ulim = 0,975 fuer conf = 0,95
    #Der erste Teil findet einen unteren und einen oberen
    #Wert, von dem aus das Bisektionsverfahren gestartet 
    #wird.
    lc <- c(-tval,tval/2,tval)
    while(pt(tval,df,lc[1])<ulim) { #nonc = lc[1] = -tval
      lc <- c(lc[1]-abs(tval),lc[1],lc[3]) #veraendert: 
      #lc[1]-tval --> lc[1]-ABS(tval)
    }
    print(paste0("untere Intervallgrenze fuer Bisektionsverfahren = ",round(lc[1],4)))
    print(paste0("obere Intervallgrenze fuer Bisektionsverfahren = ",round(lc[2],4)))
    
    #Der naechste Teil findet mit Hilfe des 
    #Bisektionsverfahrens die untere Grenze des 
    #Konfidenzintervalls fuer ncp.
    diff <- 1
    while(diff > .00001) { # Abbruchkriterium
      if(pt(tval,df,lc[2])<ulim) 
        lc <- c(lc[1],(lc[1]+lc[2])/2,lc[2]) 
      else lc <- c(lc[2],(lc[2]+lc[3])/2,lc[3])
      diff <- abs(pt(tval,df,lc[2]) - ulim) # Differenz 
      #zwischen ulim und pt
    }
    ucdf <- pt(tval,df,lc[2])
    print(paste0("Nichtzentrale t-Verteilung: G(Delta_L) = p mit Delta_L = ",round(lc[2],4)," und p = ",round(ucdf,5)))
    c(lc[2],ucdf)
  }
#
hit1 <- #vgl. Smithson, M. Nonct.ssc, http://www.michaelsmithson.online/stats/CIstuff/Nonct.ssc (Aufgerufen am 04.03.2019).
  function(tval,df,conf)
  {
    llim <- (1-conf)/2
    #Der erste Teil findet einen obere Wert, von dem 
    #aus das Bisektionsverfahren gestartet wird.
    uc <- c(tval,1.5*tval,2*tval)
    while(pt(tval,df,uc[3])>llim) {
      uc <- c(uc[1],uc[3],uc[3]+abs(tval)) #veraendert: 
      #uc[3]+tval --> uc[3]+ABS(tval)

    }
    #Der naechste Teil findet mit Hilfe des 
    #Bisektionsverfahrens die obere Grenze des 
    #Konfidenzintervalls fuer ncp.
    diff <- 1
    while(diff > .00001) {
      if(pt(tval,df,uc[2])<llim) 
        uc <- c(uc[1],(uc[1]+uc[2])/2,uc[2]) 
      else uc <- c(uc[2],(uc[2]+uc[3])/2,uc[3])
      diff <- abs(pt(tval,df,uc[2]) - llim)
    }
    lcdf <- pt(tval,df,uc[2])
    print(paste0("Nichtzentrale t-Verteilung: G(Delta_U) = p mit Delta_U = ",round(uc[2],4)," und p = ",round(lcdf,5)))
    c(uc[2],lcdf)
  }
#
CId1 <-
  function(conf,N,mean,s)
  {     
    df <- N - 1
    tval <- twert1(mean,N,s)
    #Berechnung von Cohen's d
    d1 <- mean/s 
    CI_ncpL   <- lot1(tval,df,conf)[1]
    CI_ncpU<- hit1(tval,df,conf)[1]
    CI_deltaL<- CI_ncpL/(sqrt(N))
    CI_deltaU<-CI_ncpU/(sqrt(N))
    
    print(paste0("Das Konfidenzintervall fuer Delta ist [",round(CI_ncpL,4),", ",round(CI_ncpU,4),"]."))
    print(paste0("Cohen's d = ",round(d1,4)))
    print(paste0("Das Konfidenzintervall fuer delta ist [",round(CI_deltaL,4),", ",round(CI_deltaU,4),"]. Es hat eine Breite von ",
                 abs(round(CI_deltaL-CI_deltaU,4)),"."))
  }
