#'pt' berechnet im Falle einer t-Verteilung mit 
#Nichtzentralitaetsparameter nonc (default 0) 
#die Wahrscheinlichkeit, einen Wert 
#kleiner oder gleich t zu messen.
#

sAB <- #berechnet die zusammengefasste Standardabweichung
  function(n,m,sA2,sB2) #sA2 und sB2 sind die quadrierten 
    #Standardabweichungen. sA2 = s_A^2 und sB2 = s_B^2
  {
    sAB <- sqrt(((n - 1)*sA2 + (m - 1)*sB2)/(n+ m - 2))
  }

twert2 <- #berechnet den t-Wert fuer omega_0 = 0
  function(meanx,meany,n,m,sA2,sB2)
  {
    serr <- sqrt((1/m) + (1/n))*sAB(n,m,sA2,sB2)
    t <- (meanx - meany)/serr
    
    print(paste0("t-Wert = ",round(t,4)))
    t 
  }
lot2 <- #vgl. Smithson, M. Noncent.ssc, http://www.michaelsmithson.online/stats/CIstuff/Nonct.ssc (Aufgerufen am 04.03.2019). 
  function(tval,df,conf)
  {
    
    ulim <- 1 - (1-conf)/2 # ulim = 0,975 fuer conf = 0,95
    
    #Der erste Teil findet einen unteren und einen oberen
    #Wert, von dem aus das Bisektionsverfahren gestartet 
    #wird.
    lc <- c(-tval,tval/2,tval)
    while(pt(tval,df,lc[1])<ulim) { #nonc = lc[1] = -tval
      lc <- c(lc[1]-abs(tval),lc[1],lc[3]) #veraendert: 
      #lc[1]-tval --> lc[1]-ABS(tval)
    }
    print(paste0("untere Intervallgrenze fuer Bisektionsverfahren = ",round(lc[1],4)))
    print(paste0("obere Intervallgrenze fuer Bisektionsverfahren = ",round(lc[2],4)))
   
    #Der naechste Teil findet mit Hilfe des 
    #Bisektionsverfahrens die untere Grenze des 
    #Konfidenzintervalls fuer ncp.
    diff <- 1
    while(diff > .00001) { # Abbruchkriterium
      if(pt(tval,df,lc[2])<ulim) 
        lc <- c(lc[1],(lc[1]+lc[2])/2,lc[2]) 
      else lc <- c(lc[2],(lc[2]+lc[3])/2,lc[3])
      diff <- abs(pt(tval,df,lc[2]) - ulim) # Differenz 
      #zwischen ulim und pt
    }
    ucdf <- pt(tval,df,lc[2]) # ucdf = G(Delta_L)
    print(paste0("Nichtzentrale t-Verteilung: G(Delta_L) = p mit Delta_L = ",round(lc[2],4)," und p = ",round(ucdf,4)))
    c(lc[2],ucdf)
  }
#
hit2 <- #vgl. Smithson, M. Noncent.ssc,http://www.michaelsmithson.online/stats/CIstuff/Nonct.ssc (Aufgerufen am 04.03.2019). 
  function(tval,df,conf)
  {
  
    llim <- (1-conf)/2
    #Der erste Teil findet einen obere Wert, von dem 
    #aus das Bisektionsverfahren gestartet wird.
    uc <- c(tval,1.5*tval,2*tval)
    while(pt(tval,df,uc[3])>llim) {
      uc <- c(uc[1],uc[3],uc[3]+abs(tval)) #veraendert: 
      #uc[3]+tval --> uc[3]+ABS(tval)
    }
    #Der naechste Teil findet mit Hilfe des 
    #Bisektionsverfahrens die obere Grenze des 
    #Konfidenzintervalls fuer ncp.
    diff <- 1
    while(diff > .00001) {
      if(pt(tval,df,uc[2])<llim) 
        uc <- c(uc[1],(uc[1]+uc[2])/2,uc[2]) 
      else uc <- c(uc[2],(uc[2]+uc[3])/2,uc[3])
      diff <- abs(pt(tval,df,uc[2]) - llim)
    }
    lcdf <- pt(tval,df,uc[2])
    print(paste0("Nichtzentrale t-Verteilung: G(Delta_U) = p mit Delta_U = ",round(uc[2],4)," und p = ",round(lcdf,4)))
    c(uc[2],lcdf)
  }
#
CId2 <-
  function(conf,n,m,meanx,meany,sA2,sB2)
  { df <- n+m-2
    tval <- twert2(meanx,meany,n,m,sA2,sB2)
    #Berechnung von Cohen's d
    d2 <- (meanx - meany)/sAB(n,m,sA2,sB2) 
    CI_ncpL   <- lot2(tval,df,conf)[1]
    CI_ncpU<- hit2(tval,df,conf)[1]
    CI_deltaL<- CI_ncpL*(sqrt((n + m)/(n*m)))
    CI_deltaU<-CI_ncpU*(sqrt((n + m)/(n*m)))
    
    print(paste0("Das Konfidenzintervall fuer Delta ist [",round(CI_ncpL,4),", ",round(CI_ncpU,4),"]."))
    print(paste0("Cohen's d = ",round(d2,4)))
    print(paste0("Das Konfidenzintervall fuer delta ist [",round(CI_deltaL,4),", ",round(CI_deltaU,4),"]. Es hat eine Breite von ",
    abs(round(CI_deltaL-CI_deltaU,4)),"."))
  }
